#!/usr/bin/env python3
# --------------------------------------------------------------------------------------
# SPDX-FileCopyrightText: 2021 Magenta ApS <https://magenta.dk>
# SPDX-License-Identifier: MPL-2.0
# --------------------------------------------------------------------------------------
# --------------------------------------------------------------------------------------
# Imports
# --------------------------------------------------------------------------------------
from typing import List
from typing import Optional
from uuid import UUID

from pydantic import Field

from ._shared import EffectiveTime
from ramodels.base import RABase

# --------------------------------------------------------------------------------------
# Organisation implementations
# --------------------------------------------------------------------------------------


class Authority(RABase):
    urn: str
    effective_time: EffectiveTime = Field(alias="virkning")


class OrganisationProperties(RABase):
    user_key: str = Field(alias="brugervendtnoegle")
    name: str = Field(alias="organisationsnavn")
    effective_time: EffectiveTime = Field(alias="virkning")


class OrganisationAttributes(RABase):
    properties: List[OrganisationProperties] = Field(
        alias="organisationegenskaber", min_items=1, max_items=1
    )


class OrganisationValidState(RABase):
    state: str = Field("Aktiv", alias="gyldighed")
    effective_time: EffectiveTime = Field(alias="virkning")


class OrganisationStates(RABase):
    valid_state: List[OrganisationValidState] = Field(
        alias="organisationgyldighed", min_items=1, max_items=1
    )


class OrganisationRelations(RABase):
    authority: List[Authority] = Field(alias="myndighed", min_items=1, max_items=1)


# --------------------------------------------------------------------------------------
# Organisation model
# --------------------------------------------------------------------------------------


class Organisation(RABase):
    attributes: OrganisationAttributes = Field(alias="attributter")
    states: OrganisationStates = Field(alias="tilstande")
    relations: Optional[OrganisationRelations] = Field(alias="relationer")
    # TODO, PENDING: https://github.com/samuelcolvin/pydantic/pull/2231
    # for now, this value is included,
    # and has to be excluded manually when converting to json
    uuid: Optional[UUID] = None  # Field(None, exclude=True)

    # TODO: This should be done with validators setting dynamic fields instead
    @classmethod
    def from_simplified_fields(
        cls,
        uuid: UUID,
        name: str,
        user_key: str,  # often == name,
        municipality_code: Optional[int] = None,
        date_from: str = "1930-01-01",
        date_to: str = "infinity",
    ):
        effective_time = EffectiveTime(from_date=date_from, to_date=date_to)
        attributes = OrganisationAttributes(
            properties=[
                OrganisationProperties(
                    user_key=user_key, name=name, effective_time=effective_time
                )
            ]
        )
        states = OrganisationStates(
            valid_state=[OrganisationValidState(effective_time=effective_time)]
        )

        relations = None
        if municipality_code:
            relations = OrganisationRelations(
                authority=[
                    Authority(
                        urn=f"urn:dk:kommune:{municipality_code}",
                        effective_time=effective_time,
                    )
                ]
            )

        return cls(
            attributes=attributes,
            states=states,
            relations=relations,
            uuid=uuid,
        )



#!/usr/bin/env python3
# --------------------------------------------------------------------------------------
# SPDX-FileCopyrightText: 2021 Magenta ApS <https://magenta.dk>
# SPDX-License-Identifier: MPL-2.0
# --------------------------------------------------------------------------------------
# --------------------------------------------------------------------------------------
# Imports
# --------------------------------------------------------------------------------------
from typing import List
from typing import Literal
from typing import Optional
from uuid import UUID

from pydantic import Field

from ._shared import EffectiveTime
from ._shared import Published
from ._shared import Responsible
from ramodels.base import RABase

# --------------------------------------------------------------------------------------
# Facet implementations
# --------------------------------------------------------------------------------------


class FacetProperties(RABase):
    user_key: str = Field(alias="brugervendtnoegle")
    effective_time: EffectiveTime = Field(alias="virkning")


class FacetAttributes(RABase):
    properties: List[FacetProperties] = Field(
        alias="facetegenskaber", min_items=1, max_items=1
    )


class FacetStates(RABase):
    published_state: List[Published] = Field(
        alias="facetpubliceret", min_items=1, max_items=1
    )


class FacetRelations(RABase):
    responsible: List[Responsible] = Field(alias="ansvarlig", min_items=1, max_items=1)


class FacetRef(RABase):
    object_type: Literal["facet"] = Field("facet", alias="objekttype")
    uuid: UUID
    effective_time: EffectiveTime = Field(alias="virkning")


# --------------------------------------------------------------------------------------
# Facet model
# --------------------------------------------------------------------------------------


class Facet(RABase):
    attributes: FacetAttributes = Field(alias="attributter")
    states: FacetStates = Field(alias="tilstande")
    relations: FacetRelations = Field(alias="relationer")
    # TODO, PENDING: https://github.com/samuelcolvin/pydantic/pull/2231
    # for now, this value is included, and has to be excluded when converted to json
    uuid: Optional[UUID] = None  # Field(None, exclude=True)

    # TODO: This should be done with validators setting dynamic fields instead
    @classmethod
    def from_simplified_fields(
        cls,
        uuid: UUID,
        user_key: str,
        organisation_uuid: UUID,
        date_from: str = "1930-01-01",
        date_to: str = "infinity",
    ):
        effective_time = EffectiveTime(from_date=date_from, to_date=date_to)
        attributes = FacetAttributes(
            properties=[
                FacetProperties(user_key=user_key, effective_time=effective_time)
            ]
        )
        states = FacetStates(published_state=[Published(effective_time=effective_time)])

        relations = FacetRelations(
            responsible=[
                Responsible(
                    uuid=organisation_uuid,
                    effective_time=effective_time,
                )
            ]
        )
        return cls(
            attributes=attributes,
            states=states,
            relations=relations,
            uuid=uuid,
        )



#!/usr/bin/env python3
# --------------------------------------------------------------------------------------
# SPDX-FileCopyrightText: 2021 Magenta ApS <https://magenta.dk>
# SPDX-License-Identifier: MPL-2.0
# --------------------------------------------------------------------------------------
# --------------------------------------------------------------------------------------
# Imports
# --------------------------------------------------------------------------------------
from typing import List
from typing import Optional
from uuid import UUID

from pydantic import Field

from ._shared import EffectiveTime
from ._shared import Published
from ._shared import Responsible
from .facet import FacetRef
from ramodels.base import RABase

# --------------------------------------------------------------------------------------
# Klasse implementations
# --------------------------------------------------------------------------------------


class KlasseProperties(RABase):
    user_key: str = Field(alias="brugervendtnoegle")
    title: str
    scope: Optional[str] = Field(None, alias="omfang")
    effective_time: EffectiveTime = Field(alias="virkning")


class KlasseAttributes(RABase):
    properties: List[KlasseProperties] = Field(
        alias="klasseegenskaber", min_items=1, max_items=1
    )


class KlasseStates(RABase):
    published_state: List[Published] = Field(
        alias="klassepubliceret", min_items=1, max_items=1
    )


# --------------------------------------------------------------------------------------
# Klasse implementations
# --------------------------------------------------------------------------------------


class KlasseRelations(RABase):
    responsible: List[Responsible] = Field(alias="ansvarlig")
    facet: List[FacetRef] = Field(alias="ansvarlig")


class Klasse(RABase):
    attributes: KlasseAttributes = Field(alias="attributter")
    states: KlasseStates = Field(alias="tilstande")
    relations: KlasseRelations = Field(alias="relationer")
    # TODO, PENDING: https://github.com/samuelcolvin/pydantic/pull/2231
    # for now, this value is included, and has to be excluded when converted to json
    uuid: Optional[UUID] = None  # Field(None, exclude=True)

    @classmethod
    def from_simplified_fields(
        cls,
        facet_uuid: UUID,  # uuid
        uuid: UUID,
        user_key: str,  # rarely used
        scope: Optional[str],
        organisation_uuid: UUID,
        title: str,
        date_from: str = "1930-01-01",
        date_to: str = "infinity",
    ):
        effective_time = EffectiveTime(from_date=date_from, to_date=date_to)
        attributes = KlasseAttributes(
            properties=[
                KlasseProperties(
                    user_key=user_key,
                    title=title,
                    scope=scope,
                    effective_time=effective_time,
                )
            ]
        )
        states = KlasseStates(
            published_state=[Published(effective_time=effective_time)]
        )

        relations = KlasseRelations(
            responsible=[
                Responsible(
                    uuid=organisation_uuid,
                    effective_time=effective_time,
                )
            ],
            facet=[
                FacetRef(
                    uuid=facet_uuid,
                    effective_time=effective_time,
                )
            ],
        )
        return cls(
            attributes=attributes,
            states=states,
            relations=relations,
            uuid=uuid,
        )



#!/usr/bin/env python3
# --------------------------------------------------------------------------------------
# SPDX-FileCopyrightText: 2021 Magenta ApS <https://magenta.dk>
# SPDX-License-Identifier: MPL-2.0
# --------------------------------------------------------------------------------------
# --------------------------------------------------------------------------------------
# Imports
# --------------------------------------------------------------------------------------
from typing import List
from typing import Literal
from typing import Optional
from uuid import UUID

from ._shared import OrgUnitRef
from ._shared import Person
from ._shared import Validity
from ramodels.base import RABase

# --------------------------------------------------------------------------------------
# Manager implementations
# --------------------------------------------------------------------------------------


class Responsibility(RABase):
    uuid: UUID


class ManagerLevel(RABase):
    uuid: UUID


class ManagerType(RABase):
    uuid: UUID


# --------------------------------------------------------------------------------------
# Manager model
# --------------------------------------------------------------------------------------


class Manager(RABase):
    type: Literal["manager"] = "manager"
    uuid: UUID
    # user_key: str
    org_unit: OrgUnitRef
    person: Person
    responsibility: List[Responsibility]
    manager_level: ManagerLevel
    manager_type: ManagerType
    validity: Validity

    @classmethod
    def from_simplified_fields(
        cls,
        uuid: UUID,
        # user_key: str,
        org_unit_uuid: UUID,
        person_uuid: UUID,
        responsibility_uuid: UUID,
        manager_level_uuid: UUID,
        manager_type_uuid: UUID,
        from_date: str = "1930-01-01",
        to_date: Optional[str] = None,
    ):
        person = Person(uuid=person_uuid)
        org_unit = OrgUnitRef(uuid=org_unit_uuid)
        responsibility = [Responsibility(uuid=responsibility_uuid)]
        manager_level = ManagerLevel(uuid=manager_level_uuid)
        manager_type = ManagerType(uuid=manager_type_uuid)
        validity = Validity(from_date=from_date, to_date=to_date)

        return cls(
            uuid=uuid,
            # user_key=        # user_key,
            org_unit=org_unit,
            person=person,
            responsibility=responsibility,
            manager_level=manager_level,
            manager_type=manager_type,
            validity=validity,
        )



#!/usr/bin/env python3
# --------------------------------------------------------------------------------------
# SPDX-FileCopyrightText: 2021 Magenta ApS <https://magenta.dk>
# SPDX-License-Identifier: MPL-2.0
# --------------------------------------------------------------------------------------
# --------------------------------------------------------------------------------------
# Imports
# --------------------------------------------------------------------------------------
from typing import Literal
from typing import Optional
from uuid import UUID

from ._shared import EngagementRef
from ._shared import OrgUnitRef
from ._shared import Person
from ._shared import Validity
from ramodels.base import RABase

# --------------------------------------------------------------------------------------
# Address implementations
# --------------------------------------------------------------------------------------


class AddressType(RABase):
    uuid: UUID


class OrganisationRef(RABase):
    uuid: UUID


class Visibility(RABase):
    uuid: UUID


# --------------------------------------------------------------------------------------
# Address model
# --------------------------------------------------------------------------------------


class Address(RABase):
    type: Literal["address"] = "address"
    uuid: UUID
    value: str
    value2: Optional[str]
    address_type: AddressType
    org: OrganisationRef
    person: Optional[Person] = None
    org_unit: Optional[OrgUnitRef] = None
    engagement: Optional[EngagementRef] = None
    validity: Validity
    visibility: Optional[Visibility] = None

    @classmethod
    def from_simplified_fields(
        cls,
        uuid: UUID,
        value: str,
        value2: Optional[str],
        address_type_uuid: UUID,
        org_uuid: UUID,
        from_date: str,
        to_date: Optional[str] = None,
        person_uuid: Optional[UUID] = None,
        org_unit_uuid: Optional[UUID] = None,
        engagement_uuid: Optional[UUID] = None,
        visibility_uuid: Optional[UUID] = None,
    ):
        address_type = AddressType(uuid=address_type_uuid)
        org = OrganisationRef(uuid=org_uuid)
        validity = Validity(from_date=from_date, to_date=to_date)
        person = None
        if person_uuid:
            person = Person(uuid=person_uuid)
        org_unit = None
        if org_unit_uuid:
            org_unit = OrgUnitRef(uuid=org_unit_uuid)
        engagement = None
        if engagement_uuid:
            engagement = EngagementRef(uuid=engagement_uuid)
        visibility = None
        if visibility_uuid:
            visibility = Visibility(uuid=visibility_uuid)
        return cls(
            uuid=uuid,
            value=value,
            value2=value2,
            address_type=address_type,
            org=org,
            person=person,
            org_unit=org_unit,
            engagement=engagement,
            visibility=visibility,
            validity=validity,
        )



#!/usr/bin/env python3
# --------------------------------------------------------------------------------------
# SPDX-FileCopyrightText: 2021 Magenta ApS <https://magenta.dk>
# SPDX-License-Identifier: MPL-2.0
# --------------------------------------------------------------------------------------
# --------------------------------------------------------------------------------------
# Imports
# --------------------------------------------------------------------------------------
from typing import Literal
from typing import Optional
from uuid import UUID

from ._shared import EngagementRef
from ._shared import OrgUnitRef
from ._shared import Person
from ._shared import Validity
from ramodels.base import RABase

# --------------------------------------------------------------------------------------
# Engagement implementations
# --------------------------------------------------------------------------------------


class JobFunction(RABase):
    uuid: UUID


class EngagementType(RABase):
    uuid: UUID


class Primary(RABase):
    uuid: UUID


class EngagementAssociationType(RABase):
    uuid: UUID


# --------------------------------------------------------------------------------------
# Engagement models
# --------------------------------------------------------------------------------------


class Engagement(RABase):
    type: Literal["engagement"] = "engagement"
    uuid: UUID
    org_unit: OrgUnitRef
    person: Person
    job_function: JobFunction
    engagement_type: EngagementType
    validity: Validity
    primary: Primary
    user_key: str
    extension_1: str = ""
    extension_2: str = ""
    extension_3: str = ""
    extension_4: str = ""
    extension_5: str = ""
    extension_6: str = ""
    extension_7: str = ""
    extension_8: str = ""
    extension_9: str = ""
    extension_10: str = ""

    @classmethod
    def from_simplified_fields(
        cls,
        uuid: UUID,
        org_unit_uuid: UUID,
        person_uuid: UUID,
        job_function_uuid: UUID,
        engagement_type_uuid: UUID,
        from_date: Optional[str],
        to_date: Optional[str],
        primary_uuid: UUID,
        user_key: str,
        extension_1: str = "",
        extension_2: str = "",
        extension_3: str = "",
        extension_4: str = "",
        extension_5: str = "",
        extension_6: str = "",
        extension_7: str = "",
        extension_8: str = "",
        extension_9: str = "",
        extension_10: str = "",
    ):
        return cls(
            uuid=uuid,
            org_unit=OrgUnitRef(uuid=org_unit_uuid),
            person=Person(uuid=person_uuid),
            job_function=JobFunction(uuid=job_function_uuid),
            engagement_type=EngagementType(uuid=engagement_type_uuid),
            validity=Validity(from_date=from_date, to_date=to_date),
            primary=Primary(uuid=primary_uuid),
            user_key=user_key,
            extension_1=extension_1,
            extension_2=extension_2,
            extension_3=extension_3,
            extension_4=extension_4,
            extension_5=extension_5,
            extension_6=extension_6,
            extension_7=extension_7,
            extension_8=extension_8,
            extension_9=extension_9,
            extension_10=extension_10,
        )


class EngagementAssociation(RABase):
    type: Literal["engagement_association"] = "engagement_association"
    uuid: UUID
    # user_key: str
    org_unit: OrgUnitRef
    engagement: EngagementRef
    engagement_association_type: EngagementAssociationType
    validity: Validity

    @classmethod
    def from_simplified_fields(
        cls,
        uuid: UUID,
        # user_key: str,
        org_unit_uuid: UUID,
        engagement_uuid: UUID,
        engagement_association_type_uuid: UUID,
        from_date: str = "1930-01-01",
        to_date: Optional[str] = None,
    ):
        validity = Validity(from_date=from_date, to_date=to_date)
        org_unit = OrgUnitRef(uuid=org_unit_uuid)
        engagement = EngagementRef(uuid=engagement_uuid)
        engagement_association_type = EngagementAssociationType(
            uuid=engagement_association_type_uuid
        )
        return cls(
            uuid=uuid,
            org_unit=org_unit,
            engagement=engagement,
            engagement_association_type=engagement_association_type,
            validity=validity,
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ramodels', 'ramodels.lora', 'ramodels.mo']

package_data = \
{'': ['*']}

install_requires = \
['pydantic==1.8.1']

setup_kwargs = {
    'name': 'ramodels',
    'version': '0.3.1',
    'description': 'Pydantic data models for OS2mo',
    'long_description': '<!--\nSPDX-FileCopyrightText: 2021 Magenta ApS <https://magenta.dk>\nSPDX-License-Identifier: MPL-2.0\n-->\n\n\n# MoLoRa Data Models\n\nMoLoRa data validation models powered by [pydantic](https://github.com/samuelcolvin/pydantic/#pydantic).\n\n## Versioning\nThis project uses [Semantic Versioning](https://semver.org/) with the following strategy:\n- MAJOR: Incompatible changes to existing data models\n- MINOR: Backwards compatible updates to existing data models OR new models added\n- PATCH: Backwards compatible bug fixes\n\n<!--\n## Getting Started\n\nTODO: README section missing!\n\n### Prerequisites\n\n\nTODO: README section missing!\n\n### Installing\n\nTODO: README section missing!\n\n## Running the tests\n\nTODO: README section missing!\n\n## Deployment\n\nTODO: README section missing!\n\n## Built With\n\nTODO: README section missing!\n\n## Authors\n\nMagenta ApS <https://magenta.dk>\n\nTODO: README section missing!\n-->\n## License\n- This project: [MPL-2.0](MPL-2.0.txt)\n- Dependencies:\n  - pydantic: [MIT](MIT.txt)\n\nThis project uses [REUSE](https://reuse.software) for licensing. All licenses can be found in the [LICENSES folder](LICENSES/) of the project.\n\n',
    'author': 'Magenta',
    'author_email': 'info@magenta.dk',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://magenta.dk/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
